import * as React from 'react';
import PrismJs from 'prismjs';
import clsx from 'clsx';

type PrismDerivedElement = HTMLPreElement;

export interface PrismProps extends Omit<React.HTMLProps<PrismDerivedElement>, 'children'> {
  code?: string;
  language?: string;
  lineNumbers?: boolean;
  maxLineNumber?: number;
  tabSize?: number;
}

export const Prism = React.forwardRef<PrismDerivedElement, PrismProps>(({
  code,
  language = 'plain',
  lineNumbers = false,
  maxLineNumber = Infinity,
  tabSize = 2,
  className,
  style,
  ...etcProps
}, forwardedRef) => {
  const [highlightedCode, setHighlightedCode] = React.useState<string>('');

  React.useEffect(() => {
    const loadHighlighter = async () => {
      if (language !== 'plain') {
        await import(`prismjs/components/prism-${language}`);
      }

      if (!code) {
        setHighlightedCode('');
        return;
      }

      if (!Number.isFinite(maxLineNumber)) {
        setHighlightedCode(
          PrismJs.highlight(
            code,
            PrismJs.languages[language],
            language,
          )
        )
        return;
      }

      setHighlightedCode(
        PrismJs.highlight(
          code,
          PrismJs.languages[language],
          language,
        ).split('\n').slice(0, maxLineNumber).join('\n')
      );
    }

    void loadHighlighter();
  }, [language, code, maxLineNumber]);

  if (!code) {
    return null;
  }


  return (
    <div
      className={clsx(
        'flex gap-2 leading-tight',
        className,
      )}
      style={style}
    >
      {lineNumbers && (
        <pre
          style={{
            padding: 0,
            overflow: 'visible',
            margin: 0,
            textAlign: 'right',
          }}
        >
          {new Array(maxLineNumber).fill(0).map((_, i) => (
            <React.Fragment key={i}>
              {i > 0 && '\n'}
              <code
                style={{
                  whiteSpace: 'nowrap',
                }}
              >
                {i + 1}
              </code>
            </React.Fragment>
          ))}
        </pre>
      )}
      <pre
        {...etcProps}
        ref={forwardedRef}
      >
        {
          language !== 'plain'
          && (
            <code
              className="prism"
              dangerouslySetInnerHTML={{
                __html: highlightedCode,
              }}
              style={{
                tabSize,
              }}
            />
          )
        }
        {
          language === 'plain'
          && (
            <code
              className="prism"
              style={{
                tabSize,
              }}
            >
              {code}
            </code>
          )
        }
      </pre>
    </div>
  )
});

Prism.displayName = 'Prism';

